// Datei: LCD.Cpp
//
// Ansteuerung der LCD-Anzeige
// im 4 Bit Daten Mode
//
// Holger Klabunde
// 03.03.2002

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <windows.h>

#include "ftd2xx.h"
#include "ftd2def.h"

#include "lcd.h"

unsigned char lcdport;

//*******************************************
// Schreibt ein Datenbyte in die LCD-Anzeige
// Schreibt das Byte in zwei 4 Bit Nibbles
//*******************************************
void LCDWriteByte(unsigned char data)
{
 int index;
 unsigned char usbbuf[6];

 unsigned char buff;

 index=0; 

 buff=data>>4;          //High Nibble des Bytes
 lcdport=lcdport&0xF0;	//Hier D0-D3 ersma lschen zum
 lcdport|=buff;	        //reinodern vom High Nibble
// FT_Write(fthandle,&lcdport,1,&written);
 usbbuf[index++]=lcdport;

// LCDCS_ON();	      //High Nibble schreiben
 lcdport|=LCD_CS;
 usbbuf[index++]=lcdport;
// LCDCS_OFF();	
 lcdport&=~LCD_CS;
 usbbuf[index++]=lcdport;

 buff=data&0x0F;        //Low Nibble des Bytes
 lcdport=lcdport&0xF0;	//D0-D3 ersma lschen zum
 lcdport|=buff;		    //reinodern vom Low Nibble
// FT_Write(fthandle,&lcdport,1,&written);
 usbbuf[index++]=lcdport;

// LCDCS_ON();	      //Low Nibble schreiben
 lcdport|=LCD_CS;
 usbbuf[index++]=lcdport;
// LCDCS_OFF();	
 lcdport&=~LCD_CS;
 usbbuf[index]=lcdport;

 FT_Write(fthandle,usbbuf,6,&written);
//Es geht wesentlich schneller ein Datenpaket zu senden
//als jedes Byte einzeln zu verschicken
}

//**************************************************
//Schreibt einen Text in die LCD-Anzeige
//**************************************************
void LCDWrite(char *__putstr)
{
 char __ps;
 
 __ps = *__putstr;
 
  while(__ps>0)
   {
    __putstr++;
    if (__ps== 0) break;
    LCDWriteByte(__ps);
    __ps = *__putstr;
   }
}

//****************************************************
// Springt auf Zeile x und Position y der LCD-Anzeige 
// Gltig fr 2 Zeilen
//****************************************************
void LCDPos(unsigned char zeile,unsigned char position)
{
 unsigned char buff;

 LCDRS_OFF(); //Umschalten auf Kontrollregister

// 0x80 wegen Display Adress Set.
// Erste Position ist bei mir 1, nicht 0.
// Offset's
// Zeile 1 -> 0x00+0x80-0x01=0x7F
// Zeile 2 -> 0x40+0x80-0x01=0xBF 

 switch(zeile)
  {
   case 1: buff=0x7F+position; break;
   case 2: buff=0xBF+position; break;
   default : break;
  }

 LCDWriteByte(buff); 
 LCDRS_ON(); //Umschalten auf Datenregister
}

//*****************************************
// Lscht die LCD Anzeige
// Cursor steht dann auf Position 1,Zeile 1
// Gilt auch fr mehrzeilige Anzeigen
//*****************************************
void LCDCls()
{
 LCDRS_OFF(); //Controlregister
 LCDWriteByte(0x01); //CLS
 LCDRS_ON(); //Umschalten auf Datenregister
}

//*********************************************
//Stellt die Funktionsweise der LCD Anzeige ein
//*********************************************
void LCDInit()
{
 int i;

 LCDRS_OFF();  //Control Register
 LCDCS_OFF();  //LCD-Anzeige anwhlen

//Beginn Software Reset
 for(i=0; i<3; i++)
  {
   lcdport=lcdport&0xF0;//D0-D3 ersma lschen
   lcdport|=0x03;	//Wert reinodern
   FT_Write(fthandle,&lcdport,1,&written);

   LCDCS_ON();	//High Nibble schreiben
   LCDCS_OFF(); //LCD-Anzeige abwhlen
   Sleep(100); //Ein bichen warten auf die Anzeige
  }
//Ende Software Reset


//Der erste Schreibzugriff nach dem Reset ist ein 8 Bit
//Zugriff!! Also nur einmal schreiben, nicht Low-High Nibble.

//Um die 4Bit Operation einzuschalten
 lcdport=lcdport&0xF0;//D0-D3 ersma lschen
 lcdport|=0x02;	//4 Bit Operation
 FT_Write(fthandle,&lcdport,1,&written);
 LCDCS_ON();	//High Nibble schreiben
 LCDCS_OFF(); //LCD-Anzeige abwhlen
 Wait();

 LCDWriteByte(0x28);    //4 Bit Operation,2 Line,5x7 Font
 Wait();
 LCDWriteByte(0x14); 	//Cursor Move,Right Shift
 Wait();

 LCDWriteByte(0x0C); 	//Display on, Cursor off ,Cursor no blink
 Wait();

 LCDWriteByte(0x06); 	//Increment, Display Freeze
 Wait();
 LCDWriteByte(0x02); 	//Cursor Home
 Wait();
 LCDRS_ON();		//LCD-Datenregister
}
